@@ -5,8 +5,6 @@
 
 var EventEmitter = require('events').EventEmitter;
 var spawn = require('child_process').spawn;
-var fs = require('fs');
-var exists = fs.existsSync;
 var path = require('path');
 var dirname = path.dirname;
 var basename = path.basename;
@@ -15,7 +13,7 @@ var basename = path.basename;
  * Expose the root command.
  */
 
-exports = module.exports = new Command;
+exports = module.exports = new Command();
 
 /**
  * Expose `Command`.
@@ -55,7 +53,7 @@ function Option(flags, description) {
  * @api private
  */
 
-Option.prototype.name = function(){
+Option.prototype.name = function() {
   return this.long
     .replace('--', '')
     .replace('no-', '');
@@ -69,9 +67,8 @@ Option.prototype.name = function(){
  * @api private
  */
 
-Option.prototype.is = function(arg){
-  return arg == this.short
-    || arg == this.long;
+Option.prototype.is = function(arg) {
+  return arg == this.short || arg == this.long;
 };
 
 /**
@@ -114,28 +111,28 @@ Command.prototype.__proto__ = EventEmitter.prototype;
  *        .option('-C, --chdir <path>', 'change the working directory')
  *        .option('-c, --config <path>', 'set config path. defaults to ./deploy.conf')
  *        .option('-T, --no-tests', 'ignore test hook')
- *     
+ *
  *      program
  *        .command('setup')
  *        .description('run remote setup commands')
- *        .action(function(){
+ *        .action(function() {
  *          console.log('setup');
  *        });
- *     
+ *
  *      program
  *        .command('exec <cmd>')
  *        .description('run the given remote command')
- *        .action(function(cmd){
+ *        .action(function(cmd) {
  *          console.log('exec "%s"', cmd);
  *        });
- *     
+ *
  *      program
  *        .command('*')
  *        .description('deploy the given env')
- *        .action(function(env){
+ *        .action(function(env) {
  *          console.log('deploying "%s"', env);
  *        });
- *     
+ *
  *      program.parse(process.argv);
   *
  * @param {String} name
@@ -144,7 +141,7 @@ Command.prototype.__proto__ = EventEmitter.prototype;
  * @api public
  */
 
-Command.prototype.command = function(name, desc){
+Command.prototype.command = function(name, desc) {
   var args = name.split(/ +/);
   var cmd = new Command(args.shift());
   if (desc) cmd.description(desc);
@@ -178,10 +175,10 @@ Command.prototype.addImplicitHelpCommand = function() {
  * @api public
  */
 
-Command.prototype.parseExpectedArgs = function(args){
+Command.prototype.parseExpectedArgs = function(args) {
   if (!args.length) return;
   var self = this;
-  args.forEach(function(arg){
+  args.forEach(function(arg) {
     switch (arg[0]) {
       case '<':
         self._args.push({ required: true, name: arg.slice(1, -1) });
@@ -202,7 +199,7 @@ Command.prototype.parseExpectedArgs = function(args){
  *      program
  *        .command('help')
  *        .description('display verbose help')
- *        .action(function(){
+ *        .action(function() {
  *           // output help here
  *        });
  *
@@ -211,32 +208,34 @@ Command.prototype.parseExpectedArgs = function(args){
  * @api public
  */
 
-Command.prototype.action = function(fn){
+Command.prototype.action = function(fn) {
   var self = this;
-  this.parent.on(this._name, function(args, unknown){    
+  var listener = function(args, unknown) {
     // Parse any so-far unknown options
+    args = args || [];
     unknown = unknown || [];
+
     var parsed = self.parseOptions(unknown);
-    
+
     // Output help if necessary
     outputHelpIfNecessary(self, parsed.unknown);
-    
-    // If there are still any unknown options, then we simply 
+
+    // If there are still any unknown options, then we simply
     // die, unless someone asked for help, in which case we give it
     // to them, and then we die.
-    if (parsed.unknown.length > 0) {      
+    if (parsed.unknown.length > 0) {
       self.unknownOption(parsed.unknown[0]);
     }
-    
+
     // Leftover arguments need to be pushed back. Fixes issue #56
     if (parsed.args.length) args = parsed.args.concat(args);
-    
-    self._args.forEach(function(arg, i){
+
+    self._args.forEach(function(arg, i) {
       if (arg.required && null == args[i]) {
         self.missingArgument(arg.name);
       }
     });
-    
+
     // Always append ourselves to the end of the arguments,
     // to make sure we match the number of arguments the user
     // expects
@@ -245,15 +244,17 @@ Command.prototype.action = function(fn){
     } else {
       args.push(self);
     }
-    
-    fn.apply(this, args);
-  });
+
+    fn.apply(self, args);
+  };
+  this.parent.on(this._name, listener);
+  if (this._alias) this.parent.on(this._alias, listener);
   return this;
 };
 
 /**
  * Define option with `flags`, `description` and optional
- * coercion `fn`. 
+ * coercion `fn`.
  *
  * The `flags` string should contain both the short and long flags,
  * separated by comma, a pipe or space. The following are all valid
@@ -272,7 +273,7 @@ Command.prototype.action = function(fn){
  *     program.pepper
  *     // => Boolean
  *
- *     // simple boolean defaulting to false
+ *     // simple boolean defaulting to true
  *     program.option('-C, --no-cheese', 'remove cheese');
  *
  *     program.cheese
@@ -280,7 +281,7 @@ Command.prototype.action = function(fn){
  *
  *     --no-cheese
  *     program.cheese
- *     // => true
+ *     // => false
  *
  *     // required argument
  *     program.option('-C, --chdir <path>', 'change the working directory');
@@ -300,14 +301,17 @@ Command.prototype.action = function(fn){
  * @api public
  */
 
-Command.prototype.option = function(flags, description, fn, defaultValue){
+Command.prototype.option = function(flags, description, fn, defaultValue) {
   var self = this
     , option = new Option(flags, description)
     , oname = option.name()
     , name = camelcase(oname);
 
   // default as 3rd arg
-  if ('function' != typeof fn) defaultValue = fn, fn = null;
+  if (typeof fn != 'function') {
+    defaultValue = fn;
+    fn = null;
+  }
 
   // preassign default value only for --no-*, [optional], or <required>
   if (false == option.bool || option.optional || option.required) {
@@ -322,9 +326,11 @@ Command.prototype.option = function(flags, description, fn, defaultValue){
 
   // when it's passed assign the value
   // and conditionally invoke the callback
-  this.on(oname, function(val){
+  this.on(oname, function(val) {
     // coercion
-    if (null != val && fn) val = fn(val);
+    if (null !== val && fn) val = fn(val, undefined === self[name]
+      ? defaultValue
+      : self[name]);
 
     // unassigned or bool
     if ('boolean' == typeof self[name] || 'undefined' == typeof self[name]) {
@@ -353,7 +359,7 @@ Command.prototype.option = function(flags, description, fn, defaultValue){
  * @api public
  */
 
-Command.prototype.parse = function(argv){
+Command.prototype.parse = function(argv) {
   // implicit help
   if (this.executables) this.addImplicitHelpCommand();
 
@@ -361,17 +367,19 @@ Command.prototype.parse = function(argv){
   this.rawArgs = argv;
 
   // guess name
-  this._name = this._name || basename(argv[1]);
+  this._name = this._name || basename(argv[1], '.js');
 
   // process argv
   var parsed = this.parseOptions(this.normalize(argv.slice(2)));
   var args = this.args = parsed.args;
- 
+
   var result = this.parseArgs(this.args, parsed.unknown);
 
   // executable sub-commands
   var name = result.args[0];
-  if (this._execs[name]) return this.executeSubCommand(argv, args, parsed.unknown);
+  if (this._execs[name] && typeof this._execs[name] != "function") {
+    return this.executeSubCommand(argv, args, parsed.unknown);
+  }
 
   return result;
 };
@@ -399,15 +407,16 @@ Command.prototype.executeSubCommand = function(argv, args, unknown) {
 
   // executable
   var dir = dirname(argv[1]);
-  var bin = basename(argv[1]) + '-' + args[0];
+  var bin = basename(argv[1], '.js') + '-' + args[0];
 
   // check for ./<bin> first
   var local = path.join(dir, bin);
 
   // run it
   args = args.slice(1);
-  var proc = spawn(local, args, { stdio: 'inherit', customFds: [0, 1, 2] });
-  proc.on('error', function(err){
+  args.unshift(local);
+  var proc = spawn('node', args, { stdio: 'inherit', customFds: [0, 1, 2] });
+  proc.on('error', function(err) {
     if (err.code == "ENOENT") {
       console.error('\n  %s(1) does not exist, try --help\n', bin);
     } else if (err.code == "EACCES") {
@@ -428,15 +437,26 @@ Command.prototype.executeSubCommand = function(argv, args, unknown) {
  * @api private
  */
 
-Command.prototype.normalize = function(args){
+Command.prototype.normalize = function(args) {
   var ret = []
     , arg
+    , lastOpt
     , index;
 
   for (var i = 0, len = args.length; i < len; ++i) {
     arg = args[i];
-    if (arg.length > 1 && '-' == arg[0] && '-' != arg[1]) {
-      arg.slice(1).split('').forEach(function(c){
+    if (i > 0) {
+      lastOpt = this.optionFor(args[i-1]);
+    }
+
+    if (arg === '--') {
+      // Honor option terminator
+      ret = ret.concat(args.slice(i));
+      break;
+    } else if (lastOpt && lastOpt.required) {
+      ret.push(arg);
+    } else if (arg.length > 1 && '-' == arg[0] && '-' != arg[1]) {
+      arg.slice(1).split('').forEach(function(c) {
         ret.push('-' + c);
       });
     } else if (/^--/.test(arg) && ~(index = arg.indexOf('='))) {
@@ -461,10 +481,8 @@ Command.prototype.normalize = function(args){
  * @api private
  */
 
-Command.prototype.parseArgs = function(args, unknown){
-  var cmds = this.commands
-    , len = cmds.length
-    , name;
+Command.prototype.parseArgs = function(args, unknown) {
+  var name;
 
   if (args.length) {
     name = args[0];
@@ -475,10 +493,10 @@ Command.prototype.parseArgs = function(args, unknown){
     }
   } else {
     outputHelpIfNecessary(this, unknown);
-    
+
     // If there were no args and we have unknown options,
     // then they are extraneous and we need to error.
-    if (unknown.length > 0) {      
+    if (unknown.length > 0) {
       this.unknownOption(unknown[0]);
     }
   }
@@ -494,7 +512,7 @@ Command.prototype.parseArgs = function(args, unknown){
  * @api private
  */
 
-Command.prototype.optionFor = function(arg){
+Command.prototype.optionFor = function(arg) {
   for (var i = 0, len = this.options.length; i < len; ++i) {
     if (this.options[i].is(arg)) {
       return this.options[i];
@@ -511,7 +529,7 @@ Command.prototype.optionFor = function(arg){
  * @api public
  */
 
-Command.prototype.parseOptions = function(argv){
+Command.prototype.parseOptions = function(argv) {
   var args = []
     , len = argv.length
     , literal
@@ -544,7 +562,6 @@ Command.prototype.parseOptions = function(argv){
       if (option.required) {
         arg = argv[++i];
         if (null == arg) return this.optionMissingArgument(option);
-        if ('-' == arg[0] && '-' != arg) return this.optionMissingArgument(option, arg);
         this.emit(option.name(), arg);
       // optional arg
       } else if (option.optional) {
@@ -561,11 +578,11 @@ Command.prototype.parseOptions = function(argv){
       }
       continue;
     }
-    
+
     // looks like an option
     if (arg.length > 1 && '-' == arg[0]) {
       unknownOptions.push(arg);
-      
+
       // If the next argument looks like it might be
       // an argument for this option, we pass it on.
       // If it isn't, then it'll simply be ignored
@@ -574,14 +591,31 @@ Command.prototype.parseOptions = function(argv){
       }
       continue;
     }
-    
+
     // arg
     args.push(arg);
   }
-  
+
   return { args: args, unknown: unknownOptions };
 };
 
+/**
+ * Return an object containing options as key-value pairs
+ *
+ * @return {Object}
+ * @api public
+ */
+Command.prototype.opts = function() {
+  var result = {}
+    , len = this.options.length;
+
+  for (var i = 0 ; i < len; i++) {
+    var key = this.options[i].name();
+    result[key] = key === 'version' ? this._version : this[key];
+  }
+  return result;
+};
+
 /**
  * Argument `name` is missing.
  *
@@ -589,7 +623,7 @@ Command.prototype.parseOptions = function(argv){
  * @api private
  */
 
-Command.prototype.missingArgument = function(name){
+Command.prototype.missingArgument = function(name) {
   console.error();
   console.error("  error: missing required argument `%s'", name);
   console.error();
@@ -604,7 +638,7 @@ Command.prototype.missingArgument = function(name){
  * @api private
  */
 
-Command.prototype.optionMissingArgument = function(option, flag){
+Command.prototype.optionMissingArgument = function(option, flag) {
   console.error();
   if (flag) {
     console.error("  error: option `%s' argument missing, got `%s'", option.flags, flag);
@@ -622,7 +656,7 @@ Command.prototype.optionMissingArgument = function(option, flag){
  * @api private
  */
 
-Command.prototype.unknownOption = function(flag){
+Command.prototype.unknownOption = function(flag) {
   console.error();
   console.error("  error: unknown option `%s'", flag);
   console.error();
@@ -642,32 +676,46 @@ Command.prototype.unknownOption = function(flag){
  * @api public
  */
 
-Command.prototype.version = function(str, flags){
+Command.prototype.version = function(str, flags) {
   if (0 == arguments.length) return this._version;
   this._version = str;
   flags = flags || '-V, --version';
   this.option(flags, 'output the version number');
-  this.on('version', function(){
-    console.log(str);
+  this.on('version', function() {
+    process.stdout.write(str + '\n');
     process.exit(0);
   });
   return this;
 };
 
 /**
- * Set the description `str`.
+ * Set the description to `str`.
  *
  * @param {String} str
  * @return {String|Command}
  * @api public
  */
 
-Command.prototype.description = function(str){
+Command.prototype.description = function(str) {
   if (0 == arguments.length) return this._description;
   this._description = str;
   return this;
 };
 
+/**
+ * Set an alias for the command
+ *
+ * @param {String} alias
+ * @return {String|Command}
+ * @api public
+ */
+
+Command.prototype.alias = function(alias) {
+  if (0 == arguments.length) return this._alias;
+  this._alias = alias;
+  return this;
+};
+
 /**
  * Set / get the command usage `str`.
  *
@@ -676,17 +724,16 @@ Command.prototype.description = function(str){
  * @api public
  */
 
-Command.prototype.usage = function(str){
-  var args = this._args.map(function(arg){
+Command.prototype.usage = function(str) {
+  var args = this._args.map(function(arg) {
     return arg.required
       ? '<' + arg.name + '>'
       : '[' + arg.name + ']';
   });
 
-  var usage = '[options'
-    + (this.commands.length ? '] [command' : '')
-    + ']'
-    + (this._args.length ? ' ' + args : '');
+  var usage = '[options]'
+    + (this.commands.length ? ' [command]' : '')
+    + (this._args.length ? ' ' + args.join(' ') : '');
 
   if (0 == arguments.length) return this._usage || usage;
   this._usage = str;
@@ -694,6 +741,18 @@ Command.prototype.usage = function(str){
   return this;
 };
 
+/**
+ * Get the name of the command
+ *
+ * @param {String} name
+ * @return {String|Command}
+ * @api public
+ */
+
+Command.prototype.name = function(name) {
+  return this._name;
+};
+
 /**
  * Return the largest option length.
  *
@@ -701,8 +760,8 @@ Command.prototype.usage = function(str){
  * @api private
  */
 
-Command.prototype.largestOptionLength = function(){
-  return this.options.reduce(function(max, option){
+Command.prototype.largestOptionLength = function() {
+  return this.options.reduce(function(max, option) {
     return Math.max(max, option.flags.length);
   }, 0);
 };
@@ -714,14 +773,13 @@ Command.prototype.largestOptionLength = function(){
  * @api private
  */
 
-Command.prototype.optionHelp = function(){
+Command.prototype.optionHelp = function() {
   var width = this.largestOptionLength();
-  
+
   // Prepend the help information
   return [pad('-h, --help', width) + '  ' + 'output usage information']
-    .concat(this.options.map(function(option){
-      return pad(option.flags, width)
-        + '  ' + option.description;
+    .concat(this.options.map(function(option) {
+      return pad(option.flags, width) + '  ' + option.description;
       }))
     .join('\n');
 };
@@ -733,26 +791,39 @@ Command.prototype.optionHelp = function(){
  * @api private
  */
 
-Command.prototype.commandHelp = function(){
+Command.prototype.commandHelp = function() {
   if (!this.commands.length) return '';
+
+  var commands = this.commands.map(function(cmd) {
+    var args = cmd._args.map(function(arg) {
+      return arg.required
+        ? '<' + arg.name + '>'
+        : '[' + arg.name + ']';
+    }).join(' ');
+
+    return [
+      cmd._name
+        + (cmd._alias
+          ? '|' + cmd._alias
+          : '')
+        + (cmd.options.length
+          ? ' [options]'
+          : '')
+        + ' ' + args
+    , cmd.description()
+    ];
+  });
+
+  var width = commands.reduce(function(max, command) {
+    return Math.max(max, command[0].length);
+  }, 0);
+
   return [
       ''
     , '  Commands:'
     , ''
-    , this.commands.map(function(cmd){
-      var args = cmd._args.map(function(arg){
-        return arg.required
-          ? '<' + arg.name + '>'
-          : '[' + arg.name + ']';
-      }).join(' ');
-
-      return pad(cmd._name
-        + (cmd.options.length 
-          ? ' [options]'
-          : '') + ' ' + args, 22)
-        + (cmd.description()
-          ? ' ' + cmd.description()
-          : '');
+    , commands.map(function(cmd) {
+      return pad(cmd[0], width) + '  ' + cmd[1];
     }).join('\n').replace(/^/gm, '    ')
     , ''
   ].join('\n');
@@ -765,10 +836,14 @@ Command.prototype.commandHelp = function(){
  * @api private
  */
 
-Command.prototype.helpInformation = function(){
+Command.prototype.helpInformation = function() {
   return [
       ''
-    , '  Usage: ' + this._name + ' ' + this.usage()
+    , '  Usage: ' + this._name
+        + (this._alias
+          ? '|' + this._alias
+          : '')
+        + ' ' + this.usage()
     , '' + this.commandHelp()
     , '  Options:'
     , ''
@@ -784,7 +859,7 @@ Command.prototype.helpInformation = function(){
  * @api public
  */
 
-Command.prototype.outputHelp = function(){
+Command.prototype.outputHelp = function() {
   process.stdout.write(this.helpInformation());
   this.emit('--help');
 };
@@ -795,7 +870,7 @@ Command.prototype.outputHelp = function(){
  * @api public
  */
 
-Command.prototype.help = function(){
+Command.prototype.help = function() {
   this.outputHelp();
   process.exit();
 };
@@ -809,7 +884,7 @@ Command.prototype.help = function(){
  */
 
 function camelcase(flag) {
-  return flag.split('-').reduce(function(str, word){
+  return flag.split('-').reduce(function(str, word) {
     return str + word[0].toUpperCase() + word.slice(1);
   });
 }
